@@ -127,29 +127,25 @@ hg_add_host_with_options(globals, hostname, ip, alive, netmask, use_max, ip_max)
  struct hg_host * host;
  int i;
 
- if(!hg_filter_host(globals, hostname, ip))
- {
-  c_hostname = malloc(strlen(hostname)+1);
-  strncpy(c_hostname, hostname, strlen(hostname)+1);
-  for(i=0;i<strlen(hostname);i++)c_hostname[i]=tolower(c_hostname[i]);
-  host = globals->host_list;
-  while(host->next)host = host->next;
-  host->next = malloc(sizeof(struct hg_host));
-  bzero(host->next, sizeof(struct hg_host));
+ c_hostname = strdup(hostname);
+ for(i=0;i<strlen(hostname);i++)c_hostname[i]=tolower(c_hostname[i]);
+ host = globals->host_list;
+ while(host->next)host = host->next;
+ host->next = malloc(sizeof(struct hg_host));
+ bzero(host->next, sizeof(struct hg_host));
  
-  host->hostname = c_hostname;
-  host->domain = hostname ? hg_name_to_domain(c_hostname):"";
-  host->cidr_netmask = netmask;
-  if(netmask != 32)printf("Error ! Bad netmask\n");
-  host->tested = 0;
-  host->alive = alive;
-  host->addr = ip;
-  host->use_max = use_max?1:0;
-  if(ip_max){
+ host->hostname = c_hostname;
+ host->domain = hostname ? hg_name_to_domain(c_hostname):"";
+ host->cidr_netmask = netmask;
+ if(netmask != 32)printf("Error ! Bad netmask\n");
+ host->tested = 0;
+ host->alive = alive;
+ host->addr = ip;
+ host->use_max = use_max?1:0;
+ if(ip_max){
   	host->max.s_addr = ip_max->s_addr;
 	host->min = cidr_get_first_ip(ip, netmask);
 	}
- }
 }
  
 void hg_add_domain(globals, domain)
