@@ -31,6 +31,8 @@ hg_init(hostname, flags)
  int flags;
 {
  struct hg_globals * globals = malloc(sizeof(struct hg_globals));
+
+ hostname = strdup(hostname);
  bzero(globals, sizeof(struct hg_globals));
  globals->flags = flags;
  globals->host_list = malloc(sizeof(struct hg_host));
@@ -40,6 +42,7 @@ hg_init(hostname, flags)
  bzero(globals->tested, sizeof(struct hg_host));
  
  hg_add_comma_delimited_hosts(globals, hostname);
+ free(hostname);
  return(globals);
 }
 
@@ -95,9 +98,20 @@ hg_next_host(globals, ip)
    if(!host->use_max)
    {
    if((globals->flags & HG_REVLOOKUP))
-     return hg_get_name_from_ip(host->addr);
-   else 
-     return(strdup(inet_ntoa(host->addr))); 
+     {
+      if(!host->hostname ||
+        (inet_addr(host->hostname) != INADDR_NONE)) 
+	 return hg_get_name_from_ip(host->addr);
+      else
+        return strdup(host->hostname);
+     }
+   else
+    {
+     if(host->hostname && (inet_addr(host->hostname) == INADDR_NONE))
+      return strdup(host->hostname);
+     else 
+      return(strdup(inet_ntoa(host->addr))); 
+     }
    }
    else
    {
