@@ -21,7 +21,7 @@
    Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Modified for use with Nessus by <jordan@mjh.teddy-net.com>
-   $Id: getopt.c,v 1.4 1999/12/24 21:39:04 jordan Exp $ */
+   $Id: getopt.c,v 1.6.2.1 2001/08/23 13:54:53 renaud Exp $ */
 
 /* NOTE!!!  AIX requires this to be the first thing in the file.
    Do not put ANYTHING before it!  */
@@ -29,14 +29,16 @@
 #pragma alloca
 #endif
 
+#include <config.h>
+
 #ifdef _WIN32
 #include "config.w32"
-#else
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #endif
 
+/***********************/
+#ifndef HAVE_GETOPT_LONG
+/***********************/
+
 #ifdef __GNUC__
 #define alloca __builtin_alloca
 #else /* not __GNUC__ */
@@ -68,8 +70,12 @@ char *alloca ();
    program understand `configure --with-gnu-libc' and omit the object files,
    it is simpler to just do this in the source for each such file.  */
 
+/************************************************/
 #if defined (_LIBC) || !defined (__GNU_LIBRARY__)
+#endif /* disabe #ifdef */
+/************************************************/
 
+#if 0 /* old stuff */
 /* This needs to come after some library #include
    to get __GNU_LIBRARY__ defined.  */
 #ifdef	__GNU_LIBRARY__
@@ -80,9 +86,16 @@ char *alloca ();
 #else	/* Not GNU C library.  */
 #define	__alloca	alloca
 #endif	/* GNU C library.  */
+#endif /* old stuff */
 
+#ifdef HAVE_ALLOCA
+#define __alloca alloca
+#endif
 #ifdef HAVE__ALLOCA
-#define alloca _alloca
+#define __alloca _alloca
+#endif
+#ifndef __alloca
+#error You loose -- alloca() is needed to compile this source
 #endif
 
 #if !defined (__STDC__) && !defined (const)
@@ -683,6 +696,9 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
   }
 }
 
+/************************************/
+#ifdef _WIN32 /* not used by Nessus */
+/************************************/
 int
 getopt (argc, argv, optstring)
      int argc;
@@ -694,8 +710,11 @@ getopt (argc, argv, optstring)
 			   (int *) 0,
 			   0);
 }
-
+/******************************************/
+#endif              /* not used by Nessus */
+#if 0                /* annihilate #endif */
 #endif	/* _LIBC or not __GNU_LIBRARY__.  */
+/******************************************/
 
 #ifdef TEST
 
@@ -768,3 +787,7 @@ main (argc, argv)
 }
 
 #endif /* TEST */
+
+/***************************/
+#endif /* HAVE_GETOPT_LONG */
+/***************************/
