@@ -6,7 +6,7 @@
  *
  *       Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *    $Id: hlst.c,v 1.16 2000/05/05 17:47:53 renaud Exp $ 
+ *    $Id: hlst.c,v 1.26 2001/01/21 14:18:26 renaud Exp $ 
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -56,6 +56,9 @@ struct _hashqueue {		/* linked list of bucket entries */
   unsigned          keylen;	/* length of current key */
   int               locked;	/* currently visited my some hash walk */
   struct _sorter *backlink;	/* there might be an index on that list */
+# ifdef ENABLE_RHLST
+  int               tranum;	/* transaction id, used for caching */
+# endif /* ENABLE_RHLST */
   char             key [1];	/* varable size key */
   /* varable length, pointer aligned  */
 } hashqueue ;
@@ -271,37 +274,68 @@ copy_hlst
   return new ;
 }
 
+
 void 
-destroy_hlst 
-  (hlst *h)
+flush_hlst 
+  (hlst *h,
+   void (*clup)(void*desc,void*,char*,unsigned),
+   void*             desc)
 {
   unsigned i;
+  hsrch *s ;
 
   /* sanity check */
   if (h == 0) return ;
 
+  if (clup == 0) {
+    clup = h->clup ;
+    desc = h->clup_state ;
+  }
+
   /* remove sorter */
   if (h->access != 0) {
     XFREE (h->access);
     h->access = 0;
   }
-
   for (i = 0; i < h->z.mod; i ++) {
     hashqueue *p = h->bucket [i] ;
     while (p != 0) {
       hashqueue *q = p->next ;
-      if (h->clup != 0  && p->contents != 0)
-	(*h->clup) (h->clup_state, p->contents, p->key, p->keylen);
+      if (clup != 0 && p->contents != 0)
+	(*clup) (desc, p->contents, p->key, p->keylen);
       XFREE (p);
       p = q ;
     }
+    /* done with this bucket */
+    h->bucket [i] = 0 ;
+  } /* for */
+
+  /* cannot visit any node, anymore */
+  for (s = h->walk; s != 0; s = s->next) {
+    s->hlist = 0 ;      /* next_hlst_search() will stop, that way */
+#   ifdef ENABLE_RHLST
+    if (s->clup != 0) { /* clean up by call back as early as possible */
+      (*s->clup)(s->clup_state);
+      s->clup = 0 ;
+    }
+#   endif
   }
+}
+
+
+void 
+destroy_hlst 
+  (hlst *h)
+{
+  /* sanity check */
+  if (h == 0) return ;
+
+  flush_hlst (h, 0, 0);
   if (h->clup != 0) 
     (*h->clup) (h->clup_state,0,0,0);
   XFREE (h);
 }
 
-
 /* ------------------------------------------------------------------------- *
  *                 public functions: manipulate slots                        *
  * ------------------------------------------------------------------------- */
@@ -316,7 +350,7 @@ find_hlst
   int inx ;
 
   /* sanity check */
-  if (h == 0) return 0;
+  if (h == 0 || key == 0) return 0;
   
   GET_HASH_AND_LEN (h, inx, len, key);
 
@@ -337,7 +371,7 @@ make_hlst
   int inx ;
 
   /* sanity check */
-  if (h == 0) return 0;
+  if (h == 0 || key == 0) return 0;
   
   GET_HASH_AND_LEN (h, inx, len, key);
 
@@ -376,7 +410,7 @@ delete_hlst
   unsigned inx ;
 
   /* sanity check */
-  if (h == 0) return 0;
+  if (h == 0 || key == 0) return 0;
   
   GET_HASH_AND_LEN (h, inx, len, key);
 
@@ -436,6 +470,52 @@ query_keylen_hlst
 }
 
 
+#ifdef ENABLE_RHLST
+int
+query_tranum_hlst
+  (void **t)
+{
+  return t == 0 
+    ? 0
+    : REVERT_FIELD_PTR (t, hashqueue, contents)->tranum
+    ;
+}
+
+int
+set_tranum_hlst
+  (void **t,
+   int    n)
+{
+  int last ;
+  if (t == 0) return 0;
+  last = REVERT_FIELD_PTR (t, hashqueue, contents)->tranum ;
+  REVERT_FIELD_PTR (t, hashqueue, contents)->tranum = n ;
+  return last;
+}
+
+
+int
+query_tranum_cont
+  (hlst *h)
+{
+  return h == 0 
+    ? 0
+    : h->tra_cnt
+    ;
+}
+
+
+int
+next_tranum_count
+  (hlst *h)
+{
+  return h == 0 
+    ? 0
+    : (++ h->tra_cnt < 0 ? (h->tra_cnt = 1) : h->tra_cnt)
+    ;
+}
+#endif /* ENABLE_RHLST */
+
 unsigned
 query_hlst_size
   (hlst *h)
@@ -446,6 +526,7 @@ query_hlst_size
     ;
 }
 
+
 /* ------------------------------------------------------------------------- *
  *                 public functions: search that list, itemwise              *
  * ------------------------------------------------------------------------- */
@@ -480,7 +561,9 @@ next_hlst_search
   if (s == 0) return 0;
 
   /* get the hash list */
-  h = s->hlist ;
+  if ((h = s->hlist) == 0)
+    /* list has been flushed */
+    return 0;
 
   if (s->ntry != 0)
     s->ntry->locked -- ;	/* release that node */
@@ -508,22 +591,38 @@ close_hlst_search
   (hsrch *s)
 {
   hsrch **U, *u;
-
   /* sanity check */
   if (s == 0) return ;
 
+  /* is this a stale walk descriptor? */
+  if (s->hlist == 0) {
+    XFREE (s);
+    return ;
+  }
   /* unlink current walk descriptor */
   U = &s->hlist->walk ;
-  while (u = *U, u != 0)
-    {
-    if (u == s) {
-      if (u->ntry != 0) /* release that node */
+  while (u = *U, u != 0) {
+    if (u == s) {        /* find the link pointer for that record */
+      if (u->ntry != 0)  /* release that particular node */
 	u->ntry->locked -- ;
-      }
-      *U = u->next ;
-      XFREE (u);
+      *U = u->next ;     /* unlink the walk descriptor */
+#     ifdef ENABLE_RHLST
+      if (u->clup != 0)  /* clean up peripheral my call back fn */
+	(*u->clup)(u->clup_state);
+#     endif
+      XFREE (u);         /* done */
       return ;
     }
+    if (u->next == u) { /* XXXXXXXXXXXXX Grrr - should not happen (jh) */
+      fprintf (stderr, 
+	       "%s (%n): [u->next == u] serious bug -- please report\n",
+	       __FILE__,
+	       __LINE__);
+      u->next = 0 ;
+      return;
+    }
+    U = &u->next;	/* set to successor link */
+  }
 }
 
 
@@ -546,12 +645,11 @@ for_hlst_do
     hashqueue *p = h->bucket [i] ;
     
     while (p != 0) {
-      
+      /* get next, the cb function could delete the current entry */
+      hashqueue *q = p->next ;
       if ((n = (*fn) (state, p->contents, p->key, p->keylen)) != 0)
 	return n;
-      
-      /* get next */
-      p = p->next ;
+      p = q ;
     }
   }
   
@@ -651,6 +749,14 @@ unsort_hlst
  *                 public functions: statistics                              *
  * ------------------------------------------------------------------------- */
 
+int
+hlst_buckets
+(hlst *h)
+{
+  return h == 0 ? -1 : h->z.mod;
+}
+
+
 typedef
 struct _hstatistics {
   struct {
