@@ -171,6 +171,9 @@ int recv_line(soc, buf, bufsiz)
       /* FIXME: We currently do not return an error, yet. 
 	 It would have to be globally checked out what the 
 	 functions do when -1 is returned. */
+#ifdef ECONNRESET
+     if((n < 0) && (errno == ECONNRESET))continue;
+#endif
      return 0 ;
     }
     if (n == 0) {
@@ -226,19 +229,26 @@ int recv_line(soc, buf, bufsiz)
  bzero(c, 2);
  while(!ok)
  {
+  int e;
   FD_ZERO(&rd);
   FD_SET(soc, &rd);
   tv.tv_sec = 5;
   tv.tv_usec = 0;
   if((select(soc+1, &rd, NULL, NULL, &tv))>0)
   {
-   if(FD_ISSET(soc, &rd) && ((recv(soc, c, 1, 0))>0))
+   if(FD_ISSET(soc, &rd) && ((e = recv(soc, c, 1, 0))>0))
    {
    count++;
    buf[i++] = c[0];
    if((!strlen(c)) || (!c[0]) || (c[0]=='\n') || (i>=bufsiz))ok=1;
    }
-   else ok = 1;
+   else 
+   {
+#ifdef ECONNRESET
+	   if((e < 0) && (errno == ECONNRESET))continue;
+#endif
+	   ok = 1;
+   }
   }
   else ok = 1;
  }
