@@ -1,5 +1,5 @@
 /* Nessuslib -- the Nessus Library
- * Copyright (C) 1998-1999 Renaud Deraison
+ * Copyright (C) 1998-2000 Renaud Deraison
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,6 +29,9 @@
 /* want version stuff */
 #include "libvers.h"
 
+
+#undef DEBUG_DIFF_SCAN
+
 char *nessuslib_version()
 {
   return strdup(VERSION);
@@ -85,7 +88,7 @@ void plug_set_see_also(desc, url)
  struct arglist * seealso = arg_get_value(desc, "SEE_ALSO");
  if(!seealso)
  {
-  seealso = emalloc(sizeof(struct arglist *));
+  seealso = emalloc(sizeof(*seealso));
   arg_add_value(desc, "SEE_ALSO",  ARG_ARGLIST, -1, seealso);
  }
  arg_add_value(seealso, "url", ARG_STRING, strlen(url), estrdup(url));
@@ -414,9 +417,14 @@ void host_add_port_proto(args, portnum, state, proto)
  int state;
  char * proto;
 {
- char * port_s = emalloc(20+strlen(proto));
+ char * port_s = emalloc(50+strlen(proto));
  sprintf(port_s, "Ports/%s/%d", proto, portnum);
- plug_set_key(args, port_s, ARG_INT, (void*)1);
+ if(!plug_get_key(args, port_s))
+ {
+  plug_set_key(args, port_s, ARG_INT, (void*)1);
+  sprintf(port_s, "/tmp/Ports/%s/%d", proto, portnum);
+  plug_set_key(args, port_s, ARG_INT, (void*)1);
+ }
  free(port_s);
 }
 
@@ -545,9 +553,97 @@ struct in_addr * plug_get_host_ip(desc)
  else return(NULL);
 }
 
-/* Pluto 24.6.00: reduced to one, and left the orig in place */
 
-void proto_post_wrapped(desc, port, proto, action, what)
+
+static void 
+mark_successful_plugin(desc)
+ struct arglist * desc;
+{
+ int id = (int)arg_get_value(desc, "ID");
+ char data[512];
+ 
+
+ bzero(data, sizeof(data));
+ sprintf(data, "Success/%d", id);
+ if(!plug_get_key(desc, data))
+  plug_set_key(desc, data, ARG_INT, (void*)1);
+ 
+ 
+ /*
+  * KB entries starting by /tmp/ are not saved
+  */
+ sprintf(data, "/tmp/Success/%d", id);
+ if(!plug_get_key(desc, data))
+  plug_set_key(desc, data, ARG_INT, (void*)1);
+  
+}
+
+static void
+mark_post(desc, action, content)
+ struct arglist * desc;
+ char * action;
+ char * content;
+{
+ char * entry_name = emalloc(strlen(action) + 50);
+ int num_post = (int)arg_get_value(desc, "NUM_POST");
+ 
+#ifdef DEBUG_DIFF_SCAN
+ printf("===>MARK_POST\n");
+#endif
+ sprintf(entry_name, "SentData/%d/%s/%d", arg_get_value(desc, "ID"), action, num_post);
+ plug_set_key(desc, entry_name, ARG_STRING, content);
+ 
+ efree(&entry_name);
+}
+
+static int
+post_sent_already(desc, action, content)
+ struct arglist * desc;
+ char * action;
+ char * content;
+{
+ char * trunc_name = emalloc(strlen(action) + 50);
+ int num_post = (int)arg_get_value(desc, "NUM_POST");
+ struct arglist * key = (struct arglist *)arg_get_value(desc, "key");
+ 
+ sprintf(trunc_name, "SentData/%d/%s/%d", arg_get_value(desc, "ID"), action,num_post);
+ while(key && key->next)
+ {
+#ifdef DEBUG_DIFF_SCAN
+  printf("%s & %s\n", trunc_name, key->name);
+#endif  
+  if(!strcmp(trunc_name, key->name))
+  {
+   char * v = key->value;
+   while((v = strchr(v, '')))v[0]='\n';
+   v = key->value;
+   while((v = strchr(v, '')))v[0]='\r';
+   
+#ifdef DEBUG_DIFF_SCAN
+   printf("Compare %s and %s\n", trunc_name, key->name);
+#endif   
+   if(banner_diff(content, key->value))
+   {
+#ifdef DEBUG_DIFF_SCAN
+    printf("DIFF\n");
+#endif   
+    efree(&trunc_name);
+    return 0;
+   }
+   else return 1;
+  }
+  key = key->next;
+ }
+ efree(&trunc_name);
+ return 0;
+}
+ 
+
+
+ 
+/* Pluto 24.6.00: reduced to one, and left the orig in place */
+void 
+proto_post_wrapped(desc, port, proto, action, what)
  struct arglist * desc;
  int port;
  const char * proto;
@@ -559,23 +655,39 @@ void proto_post_wrapped(desc, port, proto, action, what)
  char * buffer;
  int soc;
  struct arglist * globs;
- register struct arglist * hostdata;
+ struct arglist * hostdata;
  char c;
  char * naction;
  int len;
  ntp_caps* caps = arg_get_value(desc, "NTP_CAPS");
  char * cve;
+ int do_send = 1;
+ int num_post = (int)arg_get_value(desc, "NUM_POST");
+ 
+ if(!num_post)
+ {
+  arg_add_value(desc, "NUM_POST", ARG_INT, sizeof(int), (void*)1);
+  num_post = 1;
+ }
+ else
+ {
+  arg_set_value(desc, "NUM_POST", sizeof(int), (void*)++num_post);
+ }
+ 
+ 
  
  if(!action)
  {
   action = arg_get_value(desc, "DESCRIPTION");
  }
-  cve = arg_get_value(desc, "CVE_ID");
+ 
+ cve = arg_get_value(desc, "CVE_ID");
  if(!action)return;
  len = strlen(action);
  if(cve)len+=strlen(cve)+20;
  c = (char)13;
  
+
  if(!caps)return;
 
  hostdata = arg_get_value(desc, "HOSTNAME");
@@ -627,12 +739,46 @@ void proto_post_wrapped(desc, port, proto, action, what)
 	   what,
 	   (char *)arg_get_value(hostdata, "FQDN"), port, naction);
  }
- soc = (int)arg_get_value(desc, "SOCKET");
- globs = emalloc(sizeof(struct arglist));
- arg_add_value(globs, "global_socket", ARG_INT, sizeof(int), (void *)soc);
- auth_send(globs, buffer);
+ 
+ 
+ /*
+  * XXX only send data to the client if the option
+  * DIFF_SCAN is not set (or if the plugin used to
+  * not be successful [that is, if there is no Success/<id>
+  * key in the KB]).
+  */
+ if(arg_get_value(desc, "DIFF_SCAN"))
+ {
+  char str[128];
+#ifdef DEBUG_DIFF_SCAN  
+  fprintf(stderr, "** DIFF_SCAN enabled\n");
+#endif  
+  bzero(str, sizeof(str));
+  
+  if(!post_sent_already(desc, what, action))
+ 	 {
+	  mark_post(desc, what, action);
+ 	 }
+  else do_send = 0;
+ }
+#ifdef DEBUG_DIFF_SCAN
+ else fprintf(stderr, "*** DIFF_SCAN : NO\n");
+#endif 
+ 
+ if(do_send)
+ {
+  soc = (int)arg_get_value(desc, "SOCKET");
+  globs = emalloc(sizeof(struct arglist));
+  arg_add_value(globs, "global_socket", ARG_INT, sizeof(int), (void *)soc);
+  auth_send(globs, buffer);
+  arg_free_all(globs);
+ }
+ 
+ /*
+  * Mark in the KB that the plugin was sucessful
+  */
+ mark_successful_plugin(desc);
  efree(&buffer);
- arg_free_all(globs);
  efree(&naction);
  return;
 }
@@ -902,11 +1048,24 @@ scanner_add_port(args, port, proto)
  int len;
  int soc;
  struct arglist * globs;
+ int do_send = 1;
 #ifndef NESSUSNT
-   endservent();
+  endservent();
 #endif 
  
  
+ /*
+  * Diff scan stuff : if the port was known to be open,
+  * there is no need to report it again.
+  */
+ if(arg_get_value(args, "DIFF_SCAN"))
+ {
+   char * port_s = emalloc(50+strlen(proto));
+   sprintf(port_s, "Ports/%s/%d", proto, port);
+   if(plug_get_key(args, port_s))do_send = 0;
+   efree(&port_s);
+ }
+ 
  host_add_port_proto(args, port, 1, proto);
  
  len = 255 + (hn ? strlen(hn):0) + 
@@ -923,16 +1082,38 @@ scanner_add_port(args, port, proto)
   		hn, port);
   }
    
+ if(do_send)
+ {
  soc = (int)arg_get_value(args, "SOCKET");
  globs = emalloc(sizeof(struct arglist));
  arg_add_value(globs, "global_socket", ARG_INT, sizeof(int), (void *)soc);
  
  auth_send(globs, buf);
- free(buf);
  arg_free(globs);
+ }
+ free(buf);
 }
 
 
+/*
+ * plug_get_key() may fork(). We use this signal handler to kill
+ * its son in case the process which calls this function is killed
+ * itself
+ */
+#ifndef NESSUSNT
+static int _plug_get_key_son;
+
+static void 
+plug_get_key_sighand_term(int sig)
+{
+ if(_plug_get_key_son)
+ {
+  kill(_plug_get_key_son, SIGTERM);
+  _plug_get_key_son = 0;
+ }
+ exit(0);
+}
+#endif
 
 void * plug_get_key(args, name)
  struct arglist * args;
@@ -976,7 +1157,12 @@ void * plug_get_key(args, name)
      }
     else
      {
+       void* old;
+      _plug_get_key_son = pid;
+      old = signal(SIGTERM, plug_get_key_sighand_term);
       waitpid(pid,NULL, 0);
+      signal(SIGTERM, old);
+      _plug_get_key_son = 0;
     }
     value = value->next;
    }
