@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: common-stuff.c,v 1.11 2000/08/14 21:18:04 jordan Exp $
+ *   $Id: common-stuff.c,v 1.12 2001/01/02 12:11:41 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -153,6 +153,33 @@ xstrncasecmp
 
 #ifdef _WIN32
 #include "common-stuff.w32"
+#include "winsock.h"
+
+/* want winsock calls with errno variable set */
+int
+__peks_nt_send
+  (SOCKET       fd, 
+   const char *buf,
+   int         len,
+   int       flags)
+{
+  int n ;
+  if ((n = send (fd, buf, len, flags)) < 0)
+    errno = WSAGetLastError ();
+  return n;
+}
+int
+__peks_nt_recv
+  (SOCKET   fd, 
+   char   *buf,
+   int     len,
+   int   flags)
+{
+  int n ;
+  if ((n = recv (fd, buf, len, flags)) < 0)
+    errno = WSAGetLastError ();
+  return n;
+}
 
 /* get working directory */
 char *
