@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: hostnames.h,v 1.1 1999/10/18 22:24:31 jordan Exp $
+ *   $Id: hostnames.h,v 1.3 2000/08/14 21:18:04 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -25,8 +25,15 @@
 
 #ifndef __HOSTNAMES_H__
 #define __HOSTNAMES_H__
+#ifndef XPUB
+#define XPUB /* extract public interface */
+#endif
+
+XPUB 
+XPUB /* public functions in hostnames.c */
+XPUB extern char* tagresolve (const char *pattern, size_t len);
+XPUB 
 
-/* public functions in hostnames.c */
 extern const char *get_my_host_name (void) ;
 extern int hostcmp (const char *, const char *, size_t) ;
 extern int hostnomatch (const char *, const char *, size_t) ;
