@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: iostream.c,v 1.10 2000/08/17 20:33:59 jordan Exp $
+ *   $Id: iostream.c,v 1.10.2.2 2001/08/05 22:22:15 renaud Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -83,7 +83,7 @@
 static int tcp_connect (const char *, unsigned);   /* forward decl */
 static int tcp_listen  (const char *, unsigned, struct sockaddr_in*);
 static int tcp_accept  (int, struct sockaddr*);	   /* forward decl */
-int io_recv_timeout    = 20 ; /* secs */
+int io_recv_timeout    = 200 ; /* secs */
 int io_connect_timeout = 20 ; /* secs */
 int io_accept_timeout  = 20 ; /* secs */
 int io_listen_backlog  =  5 ; /* places in queue */
@@ -1482,7 +1482,7 @@ io_suspend
  * ------------------------------------------------------------------------ */
 
 int
-io_send
+__io_send
   (unsigned     fd,
    const char *buf,
    unsigned    len,
@@ -1524,6 +1524,33 @@ io_send
   return OS_SEND (fd, (char *)buf, len, flags) ;
 }
 
+int
+io_send
+  (unsigned     fd,
+   const char *buf,
+   unsigned    len,
+   unsigned  flags)
+{
+ int e;
+ int n = 0;
+ while(n < len)
+ {
+  int m = 0;
+  int size = 1024;
+  
+  while(m < size)
+  {
+   if((len - m - n) < size) size = len - m - n;
+   e = __io_send(fd, buf+m+n, size, 0);
+   if(e < 0)
+    return e;
+   m += e;
+  }
+  n+=m;
+ }
+ return n;   
+} 
+
 
 int 
 io_recv_tmo
