@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-file.c,v 1.20 2000/08/14 21:18:05 jordan Exp $
+ *   $Id: peks-file.c,v 1.47.2.3 2001/01/27 19:13:12 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -60,6 +60,11 @@ typedef struct stat struct_stat ;
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
+#ifdef USE_PTHREADS
+#ifdef HAVE_PTHREAD_H
+#include <pthread.h>
+#endif
+#endif
 
 #include "peks-internal.h"
 #include "peks-file.h"
@@ -83,6 +88,9 @@ typedef struct stat struct_stat ;
 #include "hostnames.h"
 #include "pubkey/elgamal.h"
 #include "baseXX.h"
+#include "peks-handshake.h"
+
+#define DUMP_MUTEX
 
 /* ---------------------------------------------------------------------- *
  *                      private definitions                               *
@@ -106,35 +114,63 @@ typedef struct stat struct_stat ;
 /* this can be used to test the el gamal crypto stuff */
 #undef USE_OLD_PASSWD_SCHEME  
 
+/* check for MS hidden file mode */
+#ifdef _WIN32
+# define             NEED_DOT_FILETYPE 1
+# define O_HIDDEN(x) (dot_file (x) ? M_HIDDEN : 0)
+#else
+# define O_HIDDEN(x)  0 /* unused */
+#endif
+
+/* ---------------------------------------------------------------------- *
+ *                         private variables                              *
+ * ---------------------------------------------------------------------- */
+
+#ifndef _WIN32
+static unsigned root_equiv_uid = ROOT_UID_EQUIV_LE ;
+static unsigned root_equiv_gid = ROOT_GID_EQUIV_LE ;
+static int      no_path_check = 0 ;
+#endif
+
+/* ------------------------------------------------------------------------ *
+ *               private functions:pthread support                          *
+ * ------------------------------------------------------------------------ */
+
+#undef  SYSNAME /* solaris wants it */
+#define SYSNAME "peks-file"
+#include "peks-sema.h"
+
 /* ---------------------------------------------------------------------- *
  *                      private helper functions                          *
  * ---------------------------------------------------------------------- */
 
-static unsigned
-valid_user_name
-  (const char *s)
-{
-  if (s == 0
-#ifdef USERNAME_NODGTSTART
-      || isdigit (*s) /* must not start with a digit */
+#ifdef _WIN32
+# define home_relative(f) ((f) [0] == '~' && ((f) [1] == '/' ||\
+                                              (f) [1] == '\\'))
+#else
+# define home_relative(f) ((f) [0] == '~' &&  (f) [1] == '/')
 #endif
-      )
-    return 0 ;
 
-  do {
+#ifdef NEED_DOT_FILETYPE
+static unsigned
+dot_file     /* check for a pseudo-hidden file starting with a dot */
+  (const char *path)
+{
+  const char *s ;
+  
+# ifdef _WIN32  
+  if ((s = strrchr (path, '\\')) != 0) 
+    return s [1] == '.' ;
+  if (isalpha (path [0]) && path [1] == ':' && path [2] == '.')
+    return 1;
+# endif
 
-    if (!isupper (*s) && 
-	!islower (*s) &&
-	!isdigit (*s) &&
-	strchr (USERNAME_XCHARS, *s) == 0)
-      return 0;
-    
-  } while (* ++ s != 0) ;
+  if ((s = strrchr (path, '/')) != 0)
+    return s [1] == '.' ;
 
-  return 1;
+  return path [0] == '.' ;
 }
-
-
+#endif /* NEED_DOT_FILETYPE */
 
 static unsigned
 matches_line_type  /* evaluates numeric value of 2nd field entry */
@@ -267,7 +303,7 @@ sleep_ms
 
 static void
 remove_stale_lockfile
-  (char *file)
+  (const char *file)
 {
   struct_stat stbuf;
   if (lstat (file, &stbuf) == 0 &&
@@ -276,6 +312,225 @@ remove_stale_lockfile
     unlink (file) ;
 }
 
+
+static int
+create_wrfile /* create a lock file, used to write things */
+  (const char *new,
+   const char *old) /* for checking, only */
+{
+  int fd, repeat, n;
+
+  /* create output file */
+  if ((fd = open3 
+       (new, O_WRONLY|O_EXCL|O_CREAT, 0600|O_HIDDEN(new))) != OPEN_ERROR)
+    return fd;
+  
+  /* remove a stale lock, if possible */
+  remove_stale_lockfile (new) ;
+  
+  if ((fd = open3 /* retry if we failed */
+       (new, O_WRONLY|O_EXCL|O_CREAT, 0600|O_HIDDEN(new))) != OPEN_ERROR)
+    return fd;
+  
+  repeat = 1; /* repeat (at least) once */
+  do { 
+    time_t limit = time (0) + FILE_LOCK_MAXW_SECS ;
+      
+    /* Somebody else is editing that file ?  So wait and retry
+       untill the lock file (aka new) has vanished */
+    
+    do {
+      fast_random_bytes ((char*)&n, sizeof (n));
+      sleep_ms (n % FILE_LOCK_WAITMOD_MS+1);
+    } while 
+      (time (0) <= limit && 
+       (fd = open3 
+	(new, O_WRONLY|O_EXCL|O_CREAT, 0600|O_HIDDEN(new))) == OPEN_ERROR);
+    
+    if (fd != OPEN_ERROR)
+      return fd ;
+      
+    /* just double check everything */
+    if ((fd = open (old, O_RDONLY)) != OPEN_ERROR) {
+      close (fd) ;
+      errno = errnum (FILE_IS_BUSY) ;
+      return OPEN_ERROR ;
+    }
+    
+    /* so the file <new> is at least FILE_LOCK_MAXWAIT_MS msecs 
+       old, and the file <file> does not exist - and we conclude 
+       that <new> is stale */
+    unlink (new) ;
+	
+  } while (-- repeat >= 0) ;
+      
+  if (fd == OPEN_ERROR &&
+      (fd = open3
+       (new, O_WRONLY|O_EXCL|O_CREAT, 0600|O_HIDDEN(new))) == OPEN_ERROR) {
+    errno = errnum (FILE_IS_BUSY) ;
+    return OPEN_ERROR ;
+  }
+  
+  return fd;
+}
+
+
+#ifndef _WIN32
+static int
+peks_private_access_path
+(const char *name,
+   int  curr_unused)
+{
+  char *p = 0;
+  int uid, gid, r_code = -1 ;
+
+  if (no_path_check)
+    return 0 ;
+
+  /* expand something like "~/file" */
+  if (home_relative (name)) {
+    if ((p = peks_get_homedir (name + 2)) == 0)
+      return -1;
+    name = p ;
+  }
+
+  name = ALLOCASTRDUP (name) ;
+  if (p != 0)
+    XFREE (p);
+
+  while ( (p = strrchr (name,  '/')) != 0) {
+    struct_stat stbuf ;
+
+    /* cut off the last path segment */
+    while (name < p && p [-1] == '/') -- p; p [0] = '\0';
+
+    POINT_OF_RANDOM_STACK (5) ;
+    if (name [0] == '\0')
+      break;
+
+    /* check the remaining path */
+    if (stat (name, &stbuf))
+      goto error_exit;
+
+    /* check for the group of the file */
+    if (stbuf.st_uid > root_equiv_uid && stbuf.st_uid != getuid ()) {
+      /* check for the owner of the path segment */
+      errno = errnum (UNTRUSTED_PATH_UID) ;
+      goto error_exit;
+    }
+    /* check for the group of the path segment */
+    if ((stbuf.st_mode & S_IWGRP) && 
+        /* unless strict, a global/group write permission requires
+	   that the sticky bit is set */
+	(stbuf.st_mode & S_ISVTX) == 0 &&
+	stbuf.st_gid > root_equiv_gid && stbuf.st_gid != getgid ()) {
+      errno = errnum (UNTRUSTED_PATH_GID);
+      goto error_exit;
+    }
+    if ((stbuf.st_mode & S_IWOTH) &&
+        /* unless strict, a global/group write permission requires
+	   that the sticky bit is set */
+	(stbuf.st_mode & S_ISVTX) == 0) {
+      errno = errnum (UNTRUSTED_PATH_OTH) ;
+      goto error_exit;
+    }
+  }
+  r_code = 0;
+
+ error_exit:
+  DEALLOCA (name);
+  return r_code;
+}
+#endif
+
+/* ---------------------------------------------------------------------- *
+ *                      public helper functions                           *
+ * ---------------------------------------------------------------------- */
+
+unsigned
+valid_user_name
+ (char     *buf, /* NULL, or of size at least (3 * strlen (s) + 1) */
+  const char *s)
+{
+  static const char tohex [] = "0123456789ABCDEF" ;
+
+  if (s == 0
+#ifdef USERNAME_NODGTSTART
+#if    USERNAME_NODGTSTART
+      || isdigit (*s) /* must not start with a digit */
+#endif
+#endif
+      )
+    return 0 ;
+
+  do {
+
+#   ifdef USERNAME_RCHARS
+#   ifdef USERNAME_ESCAPE
+    /* escape particular characters */
+    if (strchr (USERNAME_RCHARS, *s)) {
+      * buf ++ = USERNAME_ESCAPE ;
+      * buf ++ = tohex [(*s >> 4) & 0xf] ;
+      * buf ++ = tohex [ *s       & 0xf] ;
+      /* normalize, no two consecutive characters */
+      while (s [0] == s [1])
+	++ s ;
+      continue ;
+    }
+#   endif
+#   endif
+
+    if (
+#   ifdef USERNAME_ESCAPE
+	!(USERNAME_ESCAPE == s [0]  &&
+	  strchr     (tohex, s [1]) &&
+	  strchr     (tohex, s [2])) &&
+#   endif
+	!isupper (*s) &&
+	!islower (*s) &&
+	!isdigit (*s) &&
+	strchr (USERNAME_XCHARS, *s) == 0)
+      return 0;
+    
+    /* move */
+    if (buf != 0)
+      * buf ++ = * s ;
+    
+  } while (* ++ s != 0) ;
+
+  * buf = '\0' ;
+  return 1;
+}
+
+
+unsigned long
+peks_admin_equiv 
+  (unsigned uid,
+   unsigned gid)
+{
+# ifdef _WIN32
+  return gid == -1 ? -1 : 0;
+
+# else
+  unsigned u, g, n;
+
+  __enter_lock_semaphore () ;  /* Posix threads support */
+  u = (root_equiv_uid & 0xffff);
+  g = (root_equiv_gid & 0xffff);
+  n = no_path_check ;
+  if (gid == -1) {
+    no_path_check  = uid ;
+  } else {
+    root_equiv_uid = uid ;
+    root_equiv_uid = gid ;
+  }
+  __release_lock_semaphore () ;  /* Posix threads support */
+
+  return gid == -1 ? (n ? -1 : 0) : (u << 16) | (g & 0xffff);
+# endif
+}
+
+
 /* ---------------------------------------------------------------------- *
  *                  raw public key file i/o functions                     *
  * ---------------------------------------------------------------------- */
@@ -286,50 +541,64 @@ peks_private_access
   int     strictly)
 {
   struct_stat stbuf ;
+  int uid;
   
   if (lstat (name, &stbuf))
     return -1;
 
-#  ifdef HAVE_LSTAT
-   /* check for regular file or symlink */
-   if (S_ISLNK(stbuf.st_mode)) {
-     errno = errnum (NO_SYMLINK_WANTED) ;
-     return -1;
-   }
-#  endif
+# ifdef HAVE_LSTAT
+  /* check for regular file or symlink */
+  if (S_ISLNK(stbuf.st_mode)) {
+    errno = errnum (NO_SYMLINK_WANTED) ;
+    return -1;
+  }
+# endif
 
-#  ifndef _WIN32
-   /* check for regular file or hardlinks */
-   if (stbuf.st_nlink > 1) {
-     errno = errnum (NO_HARDLINK_WANTED) ;
-     return -1;
-   }
-#  endif /* _WIN32 */
+# ifndef _WIN32
+  /* check for regular file or hardlinks */
+  if (stbuf.st_nlink > 1) {
+    errno = errnum (NO_HARDLINK_WANTED) ;
+    return -1;
+  }
+  /* check for the owner of the file */
+  if (no_path_check == 0 && 
+      stbuf.st_uid > root_equiv_uid && stbuf.st_uid != getuid ()) {
+    errno = errnum (WRONG_OWNERSHIP);
+    return -1;
+  }
+# endif /* _WIN32 */
 
-   if (S_ISREG(stbuf.st_mode) == 0) {
-     errno = errnum (REG_FILE_WANTED) ;
-     return -1;
-   }
+  if (S_ISREG(stbuf.st_mode) == 0) {
+    errno = errnum (REG_FILE_WANTED) ;
+    return -1;
+  }
 
-   POINT_OF_RANDOM_STACK (21) ;
+  POINT_OF_RANDOM_STACK (21) ;
 
-#  if !defined (_WIN32) || defined (S_IWGRP)
-   if (stbuf.st_mode & (S_IWGRP|S_IWOTH)) {
-     errno = errnum (NOT_GROUP_WRITABLE) ;
-     return -1;
-   }
-   if (strictly && (stbuf.st_mode & S_IROTH)) {
-     errno = errnum (NOT_WORLD_READABLE) ;
-     return -1;
-   }
-   if (strictly > 1 && (stbuf.st_mode & S_IRGRP)) {
-     errno = errnum (NOT_GROUP_READABLE) ;
-     return -1;
-   }
-   POINT_OF_RANDOM (0,0);
-#  endif
+# if !defined (_WIN32) || defined (S_IWGRP)
+  if (stbuf.st_mode & (S_IWGRP|S_IWOTH)) {
+    errno = errnum (NOT_GROUP_WRITABLE) ;
+    return -1;
+  }
+  if (strictly && (stbuf.st_mode & S_IROTH)) {
+    errno = errnum (NOT_WORLD_READABLE) ;
+    return -1;
+  }
+  if (strictly > 1 && (stbuf.st_mode & S_IRGRP)) {
+    errno = errnum (NOT_GROUP_READABLE) ;
+    return -1;
+  }
+  POINT_OF_RANDOM (0,0);
+# endif
 
-   return 0;
+# ifdef _WIN32
+  return 0;
+# else
+  return no_path_check
+    ? 0
+    : peks_private_access_path (name, strictly)
+    ;
+# endif
 }
 
 /*
@@ -353,7 +622,7 @@ read_peks_key
    char *(*data)(int))
 {
   FILE *inp;
-  char read_buf [MAX_LINE_LENGTH] ;
+  char read_buf [MAX_LINE_LENGTH], *usrbuf ;
   const char *s, *t ;
   int n ;
 
@@ -364,11 +633,16 @@ read_peks_key
   }
 
   /* check the usr name */
-  if (usr != 0 && !valid_user_name (usr)) {
-    errno = errnum (INVALID_USER_NAME) ;
-    return 0 ;
+  usrbuf = ALLOCA (3 * (usr != 0 ? strlen (usr) : 0) + 1) ;
+  if (usr != 0) {
+    if (!valid_user_name (usrbuf, usr)) {
+      DEALLOCA (usrbuf);
+      errno = errnum (INVALID_USER_NAME) ;
+      return 0 ;
+    }
+    usr = usrbuf ;
   }
-
+  
   POINT_OF_RANDOM_STACK (13) ;
 
   /* set up custom memory manager */
@@ -386,6 +660,7 @@ read_peks_key
 
     /* check, whether this file is being edited */
     if ((fd = open (new, O_RDONLY)) == OPEN_ERROR) {
+      DEALLOCA (usrbuf);
       if (save == ENOENT)
 	errno = 0 ;
       return 0 ;
@@ -404,13 +679,17 @@ read_peks_key
 	     (fd = open (new, O_RDONLY)) != OPEN_ERROR);
 
     if (inp == 0 && (inp = fopen (file, "r")) == 0) {
+      DEALLOCA (usrbuf);
       errno = errnum (FILE_IS_BUSY) ;
       return 0;
     }
   }
 
   if (peks_private_access (file, 2) < 0) {
+    int n = errno ;
+    DEALLOCA (usrbuf);
     fclose (inp);
+    errno = n ;
     return 0;
   }
 
@@ -432,6 +711,7 @@ read_peks_key
 #      endif
 
       if (peks_keylen (k) > TRUE_ELG_KEY_BITS) {
+	DEALLOCA (usrbuf);
 	errno = errnum (USE_CRIPPLED_ELGKEY);
 	return 0;
       }
@@ -449,11 +729,13 @@ read_peks_key
 	  (*ACC) [n] = '\0' ;
 	}
       }
+      DEALLOCA (usrbuf);
       return k;
     }
   }
     
   /* did not find that tag */
+  DEALLOCA (usrbuf);
   errno = 0 ;
   fclose (inp);
   return 0;
@@ -468,14 +750,15 @@ save_peks_key
    const char  *tag,
    int       g_type, /* num val of 2nd entry, or -1 to disable */
    const char *file,
+   const char  *new, /* lockfile has been created, already */
    char *(*get_pwd)(int,char*(*)(int)),
    char *(*data)(int))
 {
   FILE *inp = 0, *outp = 0;
-  char read_buf [MAX_LINE_LENGTH], *new, *old, *s ;
-  unsigned len, r_code = -1 ;
+  char read_buf [MAX_LINE_LENGTH], *old, *newbuf, *s, *usrbuf ;
+  unsigned len ;
+  int n, r_code = -1, found_line = 0;
   HANDLE newfd, fd ;
-  int n ;
 # ifdef HAVE_UMASK
   int omask ;
 # endif
@@ -486,9 +769,14 @@ save_peks_key
   }
 
   /* check the usr name */
-  if (usr != 0 && !valid_user_name (usr)) {
-    errno = errnum (INVALID_USER_NAME) ;
-    return 0 ;
+  usrbuf = ALLOCA (3 * (usr != 0 ? strlen (usr) : 0) + 1) ;
+  if (usr != 0) {
+    if (!valid_user_name (usrbuf, usr)) {
+      DEALLOCA (usrbuf);
+      errno = errnum (INVALID_USER_NAME) ;
+      return -1 ;
+    }
+    usr = usrbuf ;
   }
 
 # ifdef TRUE_ELG_KEY_BITS
@@ -498,8 +786,9 @@ save_peks_key
 #   warning Restricted el gamal key lengths: save_peks_key
 #  endif
   if (key != 0 && peks_keylen (key) > TRUE_ELG_KEY_BITS) {
+    DEALLOCA (usrbuf);
     errno = errnum (USE_CRIPPLED_ELGKEY);
-    return 0;
+    return -1 ;
   }
 # endif /* TRUE_ELG_KEY_BITS */
 
@@ -513,9 +802,20 @@ save_peks_key
   POINT_OF_RANDOM_VAR (inp) ;
 
   /* create output file names for "new" and "old" */
-  new = ALLOCA ((len = strlen (file)) + 5) ;
-  strcat (strcpy (new, file), ".new") ;
-
+  newbuf = ALLOCA ((len = strlen (file)) + 5) ;
+  if (new == 0) {
+    /* a lockfile has not been created, yet */
+    strcat ((char*)(new = strcpy (newbuf, file)), ".new") ;
+    if ((newfd = create_wrfile (new, file)) == OPEN_ERROR)
+      goto get_ready_for_return;
+  } else {
+    if ((newfd = open (new, O_WRONLY)) == OPEN_ERROR) {
+      DEALLOCA (usrbuf);
+      errno = errnum (LOCKFILE_MISSING) ;
+      return -1;
+    }
+  }
+  
   POINT_OF_RANDOM_STACK (27) ;
 
   old = ALLOCA (len + 5) ;
@@ -523,57 +823,7 @@ save_peks_key
 
   POINT_OF_RANDOM_VAR (new) ;
 
-  /* create output file */
-  if ((newfd = open3
-       (new, O_WRONLY|O_EXCL|O_CREAT, 0600|M_HIDDEN)) == OPEN_ERROR) 
-    remove_stale_lockfile (new) ;
   
-  /* retry if we failed */
-  if (newfd < 0 &&
-      (newfd = open3
-       (new, O_WRONLY|O_EXCL|O_CREAT, 0600|M_HIDDEN)) == OPEN_ERROR) {
-
-    int repeat = 1;
-    do { /* repeat once */
-      
-      time_t limit = time (0) + FILE_LOCK_MAXW_SECS ;
-
-    /* Somebody else is editing that file ?  So wait and retry
-       untill the lock file (aka new) has vanished */
-    
-      do {
-	fast_random_bytes ((char*)&n, sizeof (n));
-	sleep_ms (n % FILE_LOCK_WAITMOD_MS+1);
-      } while (time (0) <= limit && 
-	       (newfd = open3
-		(new, O_WRONLY|O_EXCL|O_CREAT, 0600|M_HIDDEN)) 
-	       == OPEN_ERROR);
-      
-      if (newfd >= 0)
-	break ;
-
-      /* just double check everything */
-      if ((fd = open (file, O_RDONLY)) != OPEN_ERROR) {
-	close (fd) ;
-	errno = errnum (FILE_IS_BUSY) ;
-	goto get_ready_for_return;
-      }
-
-      /* so the file <new> is at least FILE_LOCK_MAXWAIT_MS msecs 
-	 old,  and the file <file> does not exist - and we conclude 
-	 that <new> is stale */
-      unlink (new) ;
-      
-    } while (-- repeat >= 0) ;
-
-    if (newfd == OPEN_ERROR &&
-	(newfd = open3
-	 (new, O_WRONLY|O_EXCL|O_CREAT, 0600|M_HIDDEN)) == OPEN_ERROR) {
-      errno = errnum (FILE_IS_BUSY) ;
-      goto get_ready_for_return;
-    }
-  }
-
 # ifdef HAVE_FDOPEN
   outp = fdopen (newfd, "w") ;
 # else
@@ -592,19 +842,17 @@ save_peks_key
     goto get_ready_for_return;
   }
   if (inp != 0) {
-
     /* check file permissions */
     if (peks_private_access (file, 2) < 0) 
       goto get_ready_for_return;
-  
     /* parse input file up until the tag line */
     while (fgets (read_buf, sizeof (read_buf), inp) > 0) {
-      
       if (read_buf [0] != '#') {
-
-	if (start_of_key_fields (usr, tag, read_buf, g_type) != 0)
+	/* is this the tag line, asked for? */
+	if (start_of_key_fields (usr, tag, read_buf, g_type) != 0) {
+	  found_line = 1;
 	  break ;
-      
+	}
 	/* check for input line overflow */
 	n = strlen (read_buf) ;
 	if (read_buf [n-1] != '\n') {
@@ -612,7 +860,6 @@ save_peks_key
 	  goto get_ready_for_return;
 	}
       }
-      
       /* copy to the output file */
       fputs (read_buf, outp) ;
     }
@@ -622,26 +869,22 @@ save_peks_key
 
   if (key != 0) {
     /* make string from private key, make whole keys line */
-    if ((s = make_peks_key_line (usr, tag, key, get_pwd, data)) == 0) {
+    if ((s = make_peks_key_line (usr, tag, key, get_pwd, data)) == 0)
       goto get_ready_for_return;
-    }
-
+    /* print the replacement line */
     fputs (s, outp);
 #   ifdef _WIN32
     fputc ('\r', outp) ;
 #   endif
     fputc ('\n', outp) ;
-
     POINT_OF_RANDOM_VAR (s) ;
     /* clean up after writing the keys string to file */
     XFREE (s);
   }
-
+  
   if (inp != 0) {
-
-    /* dump the rest onto file */
+    /* dump the rest onto the output file */
     while (fgets (read_buf, sizeof (read_buf), inp) > 0) {
-      
       /* check for input line overflow */
       n = strlen (read_buf) ;
       if (read_buf [n-1] != '\n' &&
@@ -649,11 +892,9 @@ save_peks_key
 	errno = errnum (KEYFILE_PARSER_ERR) ;
 	goto get_ready_for_return;
       }
-
       /* copy to the output file */
       fputs (read_buf, outp) ;
     }
-  
     fclose (inp);
   } /* if (inp != 0) */ 
 
@@ -665,15 +906,23 @@ save_peks_key
     rename (file, old) ;
   }
 
-  /* files have been closed, now */
+  /* input and output files have been closed, now */
   inp = outp = 0 ;
 
   /* install new key file */
-  if ((r_code = rename (new, file)) == 0) {
-    newfd = OPEN_ERROR ; /* mark  it closed */
-    unlink (old) ;
-  }
+  n = errno ;
+  if (rename (new, file) != 0)
+    goto get_ready_for_return;
 
+  newfd = OPEN_ERROR ; /* mark  it closed */
+  unlink (old) ;
+  
+  /* reached all without error, so check the search result */
+  if (found_line || key != 0)
+    r_code = 0;
+  else
+    errno = errnum (NO_SUCH_USER);
+ 
  get_ready_for_return:
   
   n = errno ;
@@ -681,8 +930,9 @@ save_peks_key
   if (outp  != 0) fclose (outp);
   if (newfd != OPEN_ERROR)
     unlink  (new);
-  DEALLOCA (new) ;
+  DEALLOCA (newbuf) ;
   DEALLOCA (old) ;
+  DEALLOCA (usrbuf);
 # ifdef HAVE_UMASK
   umask (omask) ; /* restore file creation mode */
 # endif
@@ -693,6 +943,54 @@ save_peks_key
  *               private passwd file management                           *
  * ---------------------------------------------------------------------- */
 
+static int /* sub function for peks_write_passwd */
+peks_save_passwd 
+  (peks_key   *upwd,
+   const char  *usr,
+   const char *host,
+   const char *file)
+{
+  peks_key *ukey = 0;
+  int n, e, fd ;
+  char *new = host != 0 ? ALLOCA (strlen (file) + 5) : 0 ;
+
+  if (host != 0) {
+    /* create a lock file */
+    strcat (strcpy (new, file), ".new") ;
+    if ((fd = create_wrfile (new, file)) == OPEN_ERROR) {
+      n = errno ;
+      DEALLOCA (new);
+      errno = n ;
+      return -1;
+    }
+    close (fd);
+
+    /* If a "user@host" spec exists, overwrite the passwd */
+    if ((ukey = read_peks_key (0, usr, host, 0, file, 0, 0)) != 0) 
+      end_peks_key (ukey) ;
+    else
+      /* If a "user" spec (without host) exists, overwrite the passwd */
+      if ((ukey = read_peks_key (0, usr, 0, 0, file, 0, 0)) != 0) {
+	host = 0 ;
+	end_peks_key (ukey) ;
+      }
+  }
+
+  /* overwrite the passwd for "user@host" */
+  n = save_peks_key (upwd, usr, host, 0, file, new, 0, 0) ;
+  if (new != 0) {
+    e = errno ;
+    DEALLOCA (new);
+    errno = e ;
+  }
+  /* created password line */
+  if (n < 0 && errno == errnum (NO_SUCH_USER) && upwd != 0)
+    return 0 ;
+    
+  return n;
+}
+
+
 static int
 peks_write_passwd 
   (const peks_key  *k,
@@ -706,7 +1004,7 @@ peks_write_passwd
 # ifdef USE_OLD_PASSWD_SCHEME
   mpz_t PUB ;
 # endif
-  char *s ;
+  char *s, *usrbuf ;
   int n ;
 
   if (usr == 0 || file == 0) {
@@ -716,8 +1014,18 @@ peks_write_passwd
 
   POINT_OF_RANDOM_STACK (5);
 
+  /* take care of entry without a host spec */
   if (passwd == 0 || k == 0) /* delete */
-    return save_peks_key (0, usr, host, 0, file, 0, 0) ;
+    return peks_save_passwd (0, usr, host, file) ;
+
+  /* check/expand the usr name */
+  usrbuf = ALLOCA (3 * strlen (usr) + 1) ;
+  if (!valid_user_name (usrbuf, usr)) {
+    DEALLOCA (usrbuf);
+    errno = errnum (INVALID_USER_NAME) ;
+    return 0 ;
+  }
+  usr = usrbuf ;
 
   s        = ALLOCA (sizeof (peks_key)) ;
   user_pwd = memset (s, 0, sizeof (peks_key)) ;
@@ -745,8 +1053,8 @@ peks_write_passwd
   user_pwd->crypt_str = strmpzEncrypt (s, (mpz_t *)&k->private);
 # endif
 
-  if (n >= 0) /* user_pwd == 0 <=> delete */
-    n = save_peks_key (user_pwd, usr, host, 0, file, 0, 0) ;
+  if (n >= 0) /* user_pwd != 0, so no delete */
+    n = peks_save_passwd (user_pwd, usr, host, file);
 
 # ifdef USE_OLD_PASSWD_SCHEME
   mpz_clear (user_pwd->modulus) ;
@@ -755,6 +1063,7 @@ peks_write_passwd
   XFREE (user_pwd->crypt_str) ;
 # endif
 
+  DEALLOCA (usrbuf);
   DEALLOCA (user_pwd) ;
   return n;
 }
@@ -842,17 +1151,16 @@ __get_pwd
 }
 
 
-#ifdef _WIN32
-# define home_relative(f) ((f) [0] == '~' && ((f) [1] == '/' ||\
-                                              (f) [1] == '\\'))
-#else
-# define home_relative(f) ((f) [0] == '~' &&  (f) [1] == '/')
-#endif
-
 /* ---------------------------------------------------------------------- *
  *               public key file functions                                *
  * ---------------------------------------------------------------------- */
 
+/* check whether a public key exists, already in a key file, return
+
+    1: if it does not exist (and possibly was created)
+    0: if it did exist
+   -1: otherwise, eg. the tag exists but the key differs (spoof alert) */
+
 int
 check_peks_sender_key
   (peks_key    *key,
@@ -913,7 +1221,7 @@ check_peks_sender_key
 
   if (create) {
     /* append the key in the key file */
-    if (save_peks_key (key, 0, host, -1, file, 0, 0) >= 0) {
+    if (save_peks_key (key, usr, host, -1, file, 0, 0, 0) >= 0) {
       errno = errnum (STORED_SERVER_KEY) ;
     } else
       r_code = -1 ;
@@ -1006,20 +1314,20 @@ read_peks_keyfile
 int
 create_peks_keyfile
   (peks_key            *key,
-   const char          *usr,
+   const char         *user,
    const char         *host,
    const char         *file,
    char *(*get_pwd)(int,char*(*)(int)),
    char *(*data)(int))
 {
-  char *s = 0, *u = 0 ;
+  char *s = 0, *u = 0, *usr = (char*)user ;
 
   errno = 0;
 
   POINT_OF_RANDOM_STACK (13) ;
   
   /* get user from password structure */
-  if (usr == 0 && (u = (char*)usr = peks_get_username ()) == 0)
+  if (usr == 0 && (u = usr = peks_get_username ()) == 0)
     return -1;
   /* get host name */
   if (host == 0 && (host = get_my_host_name ()) == 0) {
@@ -1041,7 +1349,7 @@ create_peks_keyfile
   POINT_OF_RANDOM_VAR (file) ;
 
   /* search for "<usr>@<host>: " first and "<usr>@ ", next */
-  if (save_peks_key (key, usr, host, -1, file, get_pwd, data) < 0) {
+  if (save_peks_key (key, usr, host, -1, file, 0, get_pwd, data) < 0) {
     switch (errno) {
     case errnum (WRONG_PASSPHRASE):
     case errnum (PASSPHRASE_WANTED):
@@ -1049,7 +1357,7 @@ create_peks_keyfile
       if (s != 0) XFREE (s);
       return -1;
     }
-    if (save_peks_key (key, usr, 0, -1, file, get_pwd, data) < 0)
+    if (save_peks_key (key, usr, 0, -1, file, 0, get_pwd, data) < 0)
       switch (errno) {
       case errnum (WRONG_PASSPHRASE):
       case errnum (PASSPHRASE_WANTED):
@@ -1122,6 +1430,10 @@ peks_private_key
       goto null_return ;
     }
     POINT_OF_RANDOM_VAR (key) ;
+#   ifndef _WIN32
+    if (no_path_check == 0 && peks_private_access_path (file, 2))
+      goto null_return ;
+#   endif
     if ((key = new_peks_key (keylen)) == 0) {
       sprintf (msg, peks_strerr (errnum (CANT_GEN_KEYS_S)),
 	       peks_strerr (errno));
@@ -1169,7 +1481,6 @@ peks_save_private_key
    const char      *file,
    char *(*get_pwd)(int))
 {
-
   char msg [300] ;
   char *usr, *host;
 
@@ -1184,7 +1495,7 @@ peks_save_private_key
   /* must be used in the current { frame } */
   usr  = ALLOCASTRDUP (uath) ;
   host = strchr (usr, '@') ;
-  
+
   if (host != 0) {
     * host = '\0' ;
     host ++ ;
@@ -1218,12 +1529,98 @@ peks_save_private_key
 }
 
 
+
+int
+peks_save_public_key
+  (const char      *uath,
+   peks_key         *key,
+   const char      *file)
+{
+  int n ;
+  peks_key *k;
+  char *usr = 0, *host = 0, *u = 0;
+
+  POINT_OF_RANDOM_VAR (host);
+
+  if (key == 0 || file == 0) {
+    errno = errnum (FUNCTION_ARG_ERROR) ;
+    return -1;
+  }
+  if (uath == 0)
+    uath = "" ;
+
+  usr  = ALLOCASTRDUP (uath) ;
+  host = strchr (usr, '@') ;
+  
+  if (host != 0) {
+    * host = '\0' ;
+    host ++ ;
+  } else
+    if (*uath == '\0') {
+      DEALLOCA (usr) ;
+      usr = 0 ;
+    }
+  
+  /* get user from password structure */
+  if (usr == 0 && (u = usr = (char*)peks_get_username ()) == 0)
+    return -1;
+  /* get host name, is statically provided (no implicit malloc) */
+  if (host == 0 && (host = (char*)get_my_host_name ()) == 0) {
+    n = errno ;
+    if (u != 0) XFREE (u);
+    if ((errno = n) == 0)
+      errno = errnum (NO_HOST_NAME) ;
+    return -1 ;
+  }
+
+  POINT_OF_RANDOM_STACK (11) ;
+
+  if (key->pubkey_str == 0) 
+    /* make puplic key, leave it for later use */
+    key->pubkey_str = make_public_elg_key_str (key);
+  /* create a public key */
+  if ((k = accept_public_elg_key_str (key->pubkey_str)) == 0) {
+    n = errno ;
+    if (u != 0)
+      XFREE (u);
+    errno = n ;
+    return -1 ;
+  }
+
+  if (usr != 0 && *usr == '\0')
+    usr = 0 ;
+ 
+  if (file [0] == '-' && file [1] == '\0') {
+    char* s = make_peks_key_line (usr, host, k, 0, 0);
+    /* print to standard out */
+    if (s == 0) {
+      n = -1 ;
+    } else {
+      n = 0 ;
+      puts (s);
+    }
+  } else {
+    /* store the public key in the file given (the same way as used
+       on the session negotiation or clinet authentication) */
+    n = check_peks_sender_key (k, usr, host, 1 /*create*/, file);
+  }
+  
+  if (u != 0)
+    XFREE (u);
+  end_peks_key (k);
+
+  return n < 0 ? -1 : !n ;
+}
+
+
+
 char *
 peks_edit_passwd_file
   (const peks_key *k,
    const char  *uath,
    int         count,
    const char   *pwd,
+
    const char  *file)
 {
   char *host, *usr, *t ;
@@ -1252,8 +1649,8 @@ peks_edit_passwd_file
 
   if ((k == 0 && pwd == 0) ||   /* delete request */
       (k != 0 && pwd != 0))     /* edit request */
-    t = peks_write_passwd 
-      (k, usr, host, count & 0x1f, pwd, file) < 0 ? 0 : (char*)"" ;
+    t = (peks_write_passwd 
+	 (k, usr, host, count & 0x1f, pwd, file) < 0 ?0 :pmalloc (1)) ;
   else 
     /* view request, append netmask if count == -1 */
     t = peks_read_passwd (k, usr, host, count < 0, file) ;
@@ -1308,7 +1705,7 @@ peks_delete_userkey
     }
   } else {
     end_peks_key (k) ;
-    n = save_peks_key (0, usr, host, -1, file, __get_pwd, get_pwd) ;
+    n = save_peks_key (0, usr, host, -1, file, 0, __get_pwd, get_pwd) ;
   }
 
   if (s != 0)
@@ -1349,7 +1746,7 @@ peks_delete_hostkey
     return -1 ;
   }
   end_peks_key (k) ;
-  n = save_peks_key (0, 0, host, -1, file, __get_pwd, get_pwd) ;
+  n = save_peks_key (0, 0, host, -1, file, 0, __get_pwd, get_pwd) ;
   if (s != 0)
     XFREE (s);
   return n;
