@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-file.h,v 1.5 2000/08/14 21:18:05 jordan Exp $
+ *   $Id: peks-file.h,v 1.11.2.1 2001/01/27 16:01:53 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -60,11 +60,29 @@
 #define USERNAME_XCHARS     "_-."
 #endif
 
+/* the set of valid chars that will be replaced by <escape><hex> */
+#ifndef USERNAME_RCHARS
+#define USERNAME_RCHARS     " \t"
+#endif
+#ifndef USERNAME_ESCAPE
+#define USERNAME_ESCAPE     '%'
+#endif
+
 /* a user name must not start with a digit */
 #ifndef USERNAME_NODGTSTART
 #define USERNAME_NODGTSTART  1
 #endif
 
+/* file/path checking: 
+   All user/group ids less or equal are considered admins 
+   pass -1 to disable ckecking */
+#ifndef ROOT_UID_EQUIV_LE
+#define ROOT_UID_EQUIV_LE 50
+#endif
+#ifndef ROOT_GID_EQUIV_LE
+#define ROOT_GID_EQUIV_LE 50
+#endif
+
 #ifndef XPUB
 #define XPUB /* extract public interface */
 #endif
@@ -87,6 +105,9 @@ XPUB extern char *peks_edit_passwd_file
 XPUB   (const peks_key *k, const char *usrathost,
 XPUB    int count, const char *pwd, const char *file);
 XPUB 
+XPUB extern int peks_save_public_key
+XPUB   (const char *uath, peks_key *key, const char *file) ;
+XPUB 
 XPUB extern int peks_delete_userkey 
 XPUB   (const char *usrathost, const char *file, char *(*get_pwd)(int));
 XPUB 
@@ -96,7 +117,9 @@ XPUB
 XPUB extern int peks_list_keyfile   
 XPUB   (void (*prt) (const char *), const char *file);
 XPUB 
-XPUB 
+XPUB extern unsigned long peks_admin_equiv
+XPUB   (unsigned uid, unsigned gid);
+XPUB
 
 extern peks_key *read_peks_key 
   (char **AccMask, 
@@ -105,7 +128,7 @@ extern peks_key *read_peks_key
 
 extern int save_peks_key 
   (peks_key *, const char *usr, const char *tag, int g_type, 
-   const char *file, 
+   const char *file, const char *lockfile,
    char *(*)(int,char*(*)(int)), char *(*get_pwd)(int));
 
 extern int check_peks_sender_key 
@@ -120,4 +143,8 @@ extern int create_peks_keyfile
   (peks_key *, const char *usr, const char *host, const char *file,
    char *(*)(int,char*(*)(int)), char *(*get_pwd)(int));
 
+extern unsigned valid_user_name
+  (char *buf, /* NULL or ls large as (3*strlen (s)+1) */ 
+   const char *s);
+
 #endif /* __PEKS_FILE_H__ */
