@@ -6,7 +6,7 @@
  *
  *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-setup.c,v 1.19 2000/08/14 21:18:06 jordan Exp $
+ *   $Id: peks-setup.c,v 1.20 2000/10/22 12:13:49 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -63,7 +63,7 @@
  *                  private variables                                     *
  * ---------------------------------------------------------------------- */
 
-static char *pekshome = 0;
+static char *pekshome = 0, *peksuser = 0;
 static void (* peks_keep_alive_notice) (const char*) = 0;
 
 /* ---------------------------------------------------------------------- *
@@ -158,23 +158,33 @@ char *
 peks_get_username
   (void)
 {
+  char *s ;
+
 # ifdef _WIN32
   char usr [80];
   int n = sizeof (usr) ;
+# else
+  char *usr ;
+  struct passwd *pwd ;
+# endif
+
+  if (peksuser != 0 && (s = getenv (peksuser)) != 0)
+    return PSTRDUP (s);
+
+# ifdef _WIN32
   if (GetUserName (usr, &n) == FALSE) {
     if ((errno = GetLastError ()) == 0)
       errno = errnum (NO_USER_NAME) ;
     return 0;
   }
 # else
-  char *usr ;
-  struct passwd *pwd = getpwuid (getuid ()) ;
-  if (pwd == 0 || (usr = pwd->pw_name) == 0) {
+  if ((pwd = getpwuid (getuid ())) == 0 || (usr = pwd->pw_name) == 0) {
     if (errno == 0)
       errno = errnum (NO_USER_NAME) ;
     return 0 ;
   }
 # endif
+
   return PSTRDUP (usr);
 }
 
@@ -244,6 +254,19 @@ peks_set_homevar
   __release_lock_semaphore () ;  /* Posix threads support */
 }
 
+
+void
+peks_set_uservar
+  (const char *usr)
+{
+  __enter_lock_semaphore () ;  /* Posix threads support */
+  if (peksuser != 0)
+    xfree (peksuser) ;
+  peksuser = (usr != 0 ? PSTRDUP (usr) : 0) ;
+  __release_lock_semaphore () ;  /* Posix threads support */
+}
+
+
 char *
 peks_get_dirfile
   (const char *file)
