@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-setup.h,v 1.9 2000/08/14 21:18:06 jordan Exp $
+ *   $Id: peks-setup.h,v 1.10 2000/11/07 11:19:37 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -57,6 +57,9 @@ XPUB
 XPUB /* set a custom HOME variable */
 XPUB extern void peks_set_homevar (const char *home);
 XPUB
+XPUB /* set LOGNAME variable */
+XPUB extern void peks_set_uservar (const char *usr);
+XPUB
 XPUB /* get the current user name */
 XPUB extern char *peks_get_username (void);
 XPUB
