@@ -6,7 +6,7 @@
  *
  *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-string.c,v 1.9 2000/08/14 21:18:06 jordan Exp $
+ *   $Id: peks-string.c,v 1.9.2.1 2001/05/30 15:09:58 renaud Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -206,7 +206,8 @@ peks_split_ident
   if (len == 0) 
     len = strlen (text) ;
   s = ALLOCA (len + 1) ;
-  (strncpy (s, text, len + 1)) [len] = '\0' ;
+  strncpy (s, text, len + 1);
+  s  [len] = '\0' ;
 
   if ((t = strtok (s, "/")) != 0)
     
