@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: rnd-pool.c,v 1.3 2000/08/28 08:34:05 jordan Exp $
+ *   $Id: rnd-pool.c,v 1.14.2.2 2001/06/15 21:45:11 renaud Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -59,6 +59,7 @@
 #include "rand/rnd-pool.h"
 
 #define DUMP_MUTEX
+#define DEBUG_DENSITY 1 /* comment on unexpected results */
 
 /* ------------------------------------------------------------------------- *
  *               private defs and data and structures                        *
@@ -66,7 +67,6 @@
 
 #define HASH_TYPE        "ripemd160"
 #define COMPRESSION_UNIT 3 /* an integer: 3 units are compressed to 1 */
-#define DIM(x)           (sizeof (x) / sizeof (x [0]))
 
 #ifdef  HAVE_DEV_RANDOM
 # ifndef DEV_RANDOM
@@ -79,6 +79,7 @@ static const char *dev_random = DEV_RANDOM;
 # define add_dev_randomness_len(b) (b) /* the same weight, currently */
 
 static int devrnd = -2 ;  /* try to open once, only */
+#endif /* HAVE_DEV_RANDOM */
 
 # define  WARN_LOG2(f,x)   (fputs ("WARNING: ",stderr),\
                             fprintf(stderr,f,x),\
@@ -86,7 +87,6 @@ static int devrnd = -2 ;  /* try to open once, only */
 # define  WARN_LOG3(f,x,y) (fputs ("WARNING: ",stderr),\
                             fprintf(stderr,f,x,y),\
                             fflush (stderr))
-#endif /* HAVE_DEV_RANDOM */
 
 /* #undef  RND_POOL_SIZE */
 /* #define RND_POOL_SIZE 500 */
@@ -105,37 +105,83 @@ static int uncompressed = 0;
 /* The density gives a means to check whats in the pool. It is
    calculated total 
          
-               #incoming-data - #outgoing-data 
-      ro   =   -------------------------------
-                       #pool-size
+                #in-data - #out-data - #uncompressed
+          ro  = -------------------------------------
+                           sizeof (pool)
 
-   Adding some data #delta-in and probably compresssing results in
-   the incremental formula
+   Remark: If greater 1, more data are put in the pool than ita formal
+           size. With the compressing, this tells how many input bits
+	   are at least used in order to create one output bit.
 
-               #incoming-data + #delta-in - #outgoing-data 
-      ro'  =   -------------------------------------------  =
-                            #pool-size'
+	   The #uncompressed variable reflects the fact, that the pool
+	   is filled up constantly and compressed only, when enough data
+	   are present to compress the pool altogether.
 
-               ro * #pool-size + #delta-in
-           =   ---------------------------
-                    #pool-size'
+	   One should note, that the act of compressing data only
+	   changes the #uncompressed parameter. If this parameter is
+	   small, the density does not significantly change although the
+	   bits available are less them before compressing. 
 
-   and removing some data #delta-out gives
+	   Due to the compressor block size (usually 3*20 == 60 bytes,
+	   COMPRESSION_UNIT times the value of the hash buffer), the
+	   number of bits making up an output bit is at most O(ro) with
+	   a big O constant larget than 1 dependent on the compressor
+	   algorithm only: sizeof(pool)/compressor-block-size >> 1.
 
-               #incoming-data - #outgoing-data - #delta-out
-      ro''  =   -------------------------------------------  =
-                            #pool-size'
+	   In any case, ro is a lower bound for the number of bits at
+	   least used in order to create one output bit.
 
-               ro * #pool-size - #delta-out
-           =   ----------------------------   for #pool-size'' > 0.
-                    #pool-size''
 
+   Adding some data #delta-in, leaving #uncompressed' data uncompressed
+   and probably compresssing results in the incremental formula
+
+                #in-data + #delta-in - #out-data - #uncompressed'
+          ro' = ------------------------------------------------- =
+                           sizeof (pool)
+
+                     #delta-in - #delta-uncompressed
+              = ro + -------------------------------
+                           sizeof (pool)
+
+          (where #delta-uncompressed = #uncompressed' - uncompressed)
+
+   Remark: In weird cases, adding data may decrease the density, the
+           lower bound for the number of input bits that make up an
+	   output bit.
+           
+	   Assume there are no uncompressed data in the pool and you
+	   compress it. In this case, #delta-in == 0. Due to the
+	   compressing algorithm, there might show up some data left
+	   uncompressed in the current round, giving ro < ro.
+	   
+	   One might argue, that the data left uncompressed in the
+	   current round might have been compressed by an earlier 
+	   round.  But as the compressor algorithm is thought of evenly
+	   mix and spread the compressed data, the uncompressed data
+	   left over have another density. So we just let the lower 
+	   bound decrease, the maximum decrement being significantly
+	   smaller than 1: compressor-block-size/sizeof(pool) << 1.
+
+
+   Removing some data #delta-out gives
+
+                #in-data - #out-data - #uncompressed - #delta-out 
+          ro" = -------------------------------------------------  =
+                           sizeof (pool)
+
+                            #delta-out
+              =       ro - -------------
+                           sizeof (pool)
+
+   Remark: Removing data always decreases the lower bound only,
+           although the number of input bits that make up an output bit
+	   may remain the same.
 */
 
-static float density = 1.0, maximum_density = 1e30;
+static float density = 0, maximum_density = 1e30;
 
 /* the same as above but public for easy checking */
-unsigned idensity, imax_density = -1 ;
+unsigned i100density, imax_density = (unsigned)-1 ;
 
 /* ------------------------------------------------------------------------- *
  *                   private functions                                       *
@@ -195,23 +241,27 @@ unsigned idensity, imax_density = -1 ;
  *   empty:    put_inx  > get_inx == -1
  */
 
+#if 0
 static unsigned
 pool_size (void)
 {
   int n ;
-  
-  if ((n = put_inx - get_inx) <= 0)
-    return DIM (pool) - n ;
-
+  if ((n = (put_inx - get_inx)) <= 0)
+    return sizeof (pool) + n ;
   return get_inx < 0 ? 0 : n ;
 }
+#else
+#define pool_size() (put_inx <= get_inx				\
+		     ?  sizeof (pool)   + put_inx - get_inx	\
+		     : (get_inx >= 0 ? put_inx - get_inx : 0)	)
+#endif
 
-static unsigned
+static unsigned /* returns number of bytes stored */
 put_data 
   (const char *p, 
    unsigned    n)
 {
-  int avail, stored = 0 ;
+  unsigned avail, stored = 0 ;
 
   if (n == 0)
     /* nothing to do */
@@ -221,24 +271,23 @@ put_data
   if (get_inx < put_inx) {
     
     /* store the maximum value, possible */
-    if ((stored = DIM (pool) - put_inx) > (int)n) 
+    if ((stored = sizeof (pool) - put_inx) > n) 
       stored = n ;
     memcpy (pool + put_inx, p, stored) ;
 
     /* update indices, wrap around if necessary */
-    if ((put_inx += stored) == DIM (pool))
-      put_inx = 0;
+    if ((put_inx += stored) == sizeof (pool))
+      put_inx = 0 ;  /* wrap around */
     if (get_inx < 0)
-      get_inx = 0 ;
+      get_inx = 0 ;  /* pool was empty */
 
     /* done, everything stored ? */
-    if ((n -= stored) == 0)
+    if (n == stored)
       return stored ;
 
-    /* otherwise proceed with the 2nd case */
+    /* otherwise proceed with the 2nd case, here put_inx == 0 */
+    n -= stored ;
     p += stored ;
-
-    assert (put_inx == 0);
   }
 
   /* 2nd case, calculate the space, available */
@@ -247,7 +296,7 @@ put_data
     return stored ;
 
   /* store the maximum value, possible */
-  if (avail > (int)n) 
+  if (avail > n) 
     avail = n ;
   memcpy (pool + put_inx, p, avail) ;
   
@@ -258,12 +307,12 @@ put_data
 
 
 
-static unsigned
+static unsigned  /* returns the number of data, retrieved */
 get_data 
   (char    *p, 
    unsigned n)
 {
-  int avail, retrieved = 0 ;
+  unsigned avail, retrieved = 0 ;
 
   if (n == 0)
     /* nothing to do */
@@ -273,22 +322,21 @@ get_data
   if (get_inx >= put_inx) {
 
     /* get as much as possible */
-    if ((retrieved = DIM (pool) - get_inx) > (int)n)
+    if ((retrieved = sizeof (pool) - get_inx) > n)
       retrieved = n ;
     memcpy (p, pool + get_inx, retrieved) ;
 
     /* update index, fifo might be empty, now */
-    if ((get_inx += retrieved) == DIM (pool))
+    if ((get_inx += retrieved) == sizeof (pool))
       get_inx = put_inx ? 0 : -1 ;
     
     /* done, everything loaded ? */
-    if ((n -= retrieved) == 0)
+    if (n == retrieved)
       return retrieved;
 
-    /* otherwise proceed with the 1st case */
+    /* otherwise proceed with the 1st case, here: get_inx <= 0 */
+    n -= retrieved ;
     p += retrieved ;
-
-    assert (get_inx == 0);
   }
 
   /* 1st case, normal mode, fifo may be empty */
@@ -296,13 +344,13 @@ get_data
     return retrieved ;
 
   /* get the maximum value, possible */
-  if ((avail = put_inx - get_inx) > (int)n)
+  if ((avail = (put_inx - get_inx)) > n)
     avail = n;
   memcpy (p, pool + get_inx, avail) ;
 
   /* update index, fifo may become empty */
   if ((get_inx += avail) == put_inx) {
-    put_inx = 0 ;
+    put_inx =  0;
     get_inx = -1;
   }
 
@@ -310,7 +358,7 @@ get_data
 }
 
 
-static void
+static unsigned /* returns data not compressed, yet */
 compress_data (void)
 {
   static frame_desc *md ;
@@ -324,31 +372,37 @@ compress_data (void)
 
   /* buffer size: compression unit + overlapping space */
   chunk = COMPRESSION_UNIT * md->mdlen ;
-  s     = ALLOCA (chunk + md->mdlen) ;
-  len   = pool_size ();
+
+  /* it makes no sense to compress less data */
+  if ((len = pool_size ()) < chunk)
+    return chunk - len ;
+  
+  /* allocate block compression buffer */
+  s = ALLOCA (chunk + md->mdlen);
 
   /* start: fill the overlapping space */
   len -= get_data (s + chunk,  md->mdlen) ;
 
   /* loop over the number of remaining data in the pool */
-  while (len > 0) {
+  while (len >= chunk) {
 
     /* shift left the overlapping space */
     memcpy (s, s + chunk, md->mdlen);
 
     /* append next data */
-    n = get_data (s + md->mdlen, chunk) ;
+    n = get_data (s + md->mdlen, chunk);
     
     /* hash the compression unit + overlapping space */
-    XCRCFIRST (md, s, n + md->mdlen) ;
+    XCRCFIRST (md, s, n + md->mdlen);
 
     /* replace the compressed unit */
-    put_data (XCRCRESULT0 (md), md->mdlen) ;
+    put_data (XCRCRESULT0 (md), md->mdlen);
 
     /* update counter */
     len -= n;
   }
   DEALLOCA (s);
+  return len ;
 }
 
 
@@ -379,12 +433,23 @@ make_random_bytes
  (void *p,
   int   n)
 {
+  char *pc = p;
 # ifdef HAVE_DEV_RANDOM
   if (devrnd < -1)   /* try to open once, only */
     open_random () ;
   if (devrnd >= 0) {
     /* read from random device, if possible */
-    int m = read (devrnd, p, n) ;
+    int m;
+    fd_set rd;
+    struct timeval tv = {0,10000};
+    
+    FD_ZERO(&rd);
+    FD_SET(devrnd, &rd);
+    if(select(devrnd + 1, &rd, NULL, NULL, &tv) <= 0)
+     goto lastresort;
+    
+    
+    m = read (devrnd, p, n) ;
     if (m >= n) return ;
     
     if (m >= 0 ) {
@@ -396,12 +461,13 @@ make_random_bytes
 		 dev_random, strerror (errno));
     }
   }
+lastresort:  
 # endif /* HAVE_DEV_RANDOM */
   
   /* last resort -- do your best */
-  while (n --) {       
+  while (n --) {
     /* calculate an integer smaller than 256.0 - 1/(RAND_MAX-1.0) */
-    * ((char*)p) ++ = (unsigned char)(256.0 * rand () / (RAND_MAX+1.0)) ;
+    * (pc) ++ = (unsigned char)(256.0 * rand () / (RAND_MAX+1.0)) ;
   }
 }
 
@@ -410,35 +476,56 @@ get_random_pool_data
   (char     *buf,
    unsigned  len)
 {
-  int retrieved, n, available ;
-  float old_size ;
+  int retrieved, size, available ;
 
   __enter_lock_semaphore () ;
 
-  n         = pool_size () ;
-  old_size  = (float)n ;
-  available = n - uncompressed ;
+  size      = pool_size () ;
+  available = size - uncompressed ;
 
-  assert (available >= 0);
+# ifdef DEBUG_DENSITY
+  /* which is the case that cannot happen */
+  if (available < 0) {
+    WARN_LOG3 ("get_random_pool_data: pool_size=%d < uncompressed=%d "
+	       "-- please report.\n",
+	       size, uncompressed);
+    WARN_LOG3 ("get_random_pool_data(cont): put_inx=%d, get_inx=%d "
+	       "-- please report.\n",
+	       put_inx, get_inx);
+    uncompressed = size ;
+    available    =    0 ;
+  }
+# endif
 
   /* dont let out any data compressed at least once */
-  if (n > available)
-    n = available ;
-  if (n == 0) {
+  if (available == 0) {
+#   ifdef DEBUG_DENSITY
+    if (i100density > 50) {
+      WARN_LOG3 ("get_random_pool_data: i100density=%d(%.2f)"
+		 " rounding error unexpectedly large"
+		 "-- please report.\n",
+		 i100density, density);
+    }
+#   endif
+    /* the density might be slightly positive due to rounding errors */
+    density     = 0 ;
+    i100density = 0 ;
     __release_lock_semaphore () ;
     return 0;
   }
 
+  /* deliver data from the pool */
+  if (len > available)
+    len = available ;
   retrieved = get_data (buf, len) ;
 
-  if ((n = pool_size ()) == 0)
-    density = 1.0 ;
-  else {
-    density *=  old_size ;
-    density -= retrieved ;
-    density /=         n ;
+  if ((size = pool_size ()) <= uncompressed) {
+    density     = 0 ;
+    i100density = 0 ;
+  } else {
+    density    -= (((float)retrieved) / ((float)sizeof (pool)));
+    i100density = (100.0 * density) ;
   }
-  idensity = (unsigned)(density * 100.0) ;
 
   __release_lock_semaphore () ;
   return retrieved ;
@@ -485,28 +572,40 @@ put_random_pool_data_nosync
   (const char *buf,
    unsigned    len)
 {
-  int n, processed, left;
-  float old_size ;
+  int n, processed, left, old_uncompr;
+  float delta ;
 
-  old_size  = (float)pool_size () ;
-  processed =   0;
-  left      = len;
-  n         =   0;
+  old_uncompr = uncompressed ;
+  left        =          len ;
+  processed   =            0 ;
+  n           =            0 ;
 
   while (left && (n = put_data (buf + processed, left)) < left) {
-    processed += n ;
-    left      -= n ;
-    compress_data () ;
-    uncompressed = 0 ;
+    processed   += n ;
+    left        -= n ;
+    uncompressed = compress_data () ;
   }
-  uncompressed += n ;
+  if (left) /* (left > 0) => (n == left) */
+    uncompressed += n;
   
-  density *=  old_size ;
-  density +=       len ;
-  density /= pool_size () ;
-  
-  idensity = (unsigned)(density * 100.0) ;
+  /* calculate the effect on the density */
+  if ((delta = (int)len - uncompressed + old_uncompr) == 0) 
+    return len ;
+  delta /= sizeof (pool) ;
 
+  /* the density is a lower bound, only */
+  if ((density += delta) < 0) {
+#   ifdef DEBUG_DENSITY
+    WARN_LOG3 ("put_random_pool_data: delta=%.2f ==> density=%.2f"
+	       " unexpected pool density underflow -- please report.\n",
+	       delta, density);
+#   endif
+    density     = 0 ;
+    i100density = 0 ;
+    return len ;
+  } 
+  
+  i100density = (100.0 * density) ;
   return len ;
 }
 
@@ -518,7 +617,7 @@ point_of_random_time
 {
   __enter_lock_semaphore () ;
 
-  if (idensity >= imax_density) {
+  if (i100density >= imax_density) {
 #   ifdef DEBUG_DENSITY_LEVEL
     WARN_LOG3 ("Density %.3f better than max(%.3f) -- not collecting data.\n", 
 	       density, maximum_density);
@@ -617,7 +716,7 @@ random_pool_density_percentage
 {
   unsigned n;
   __enter_lock_semaphore () ;
-  n = idensity ;
+  n = i100density ;
   __release_lock_semaphore () ;
   return n;
 }
@@ -631,7 +730,7 @@ max_random_pool_density_percentage
 
   __enter_lock_semaphore () ;
   n            = imax_density ;
-  imax_density =            d ;
+  imax_density =           d ;
   maximum_density = (float)d / 100.0 ;
   __release_lock_semaphore () ;
 
