@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-svc.c,v 1.1 2000/08/14 20:51:35 jordan Exp $
+ *   $Id: peks-svc.c,v 1.2 2000/09/23 10:39:06 renaud Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -333,7 +333,7 @@ psvc_set_proto
       continue ;
     
     /* got entry, find a matching prototype in the prototype list */
-    if (((void**)Pl = &rpc->plist, pl = *Pl) != 0) do {	
+    if ((Pl = (svc_proto**)&rpc->plist, pl = *Pl) != 0) do {	
       
       /* overwrite or remove an existing descriptor */
       if (strcmp (pl->fmt, fmt) == 0 && pl->laxen == laxen) {
@@ -368,7 +368,7 @@ psvc_set_proto
   /* append new function and prototype, *Rpc == <last-link> */
   *Rpc = rpc  = XMALLOC (sizeof (psvc_ftab)) ;
   rpc->svc   = XSTRDUP (svc) ;  
-  (void**)Pl = &rpc->plist ;
+  Pl = (svc_proto**)&rpc->plist ;
 
  set_proto_and_return:
 
@@ -784,7 +784,8 @@ psvc_import
   }
 
   if ((len = buffer2ushort ((char*)data)) != 0) {
-    fmt = (((char*)data) += 2) ;
+    
+    fmt = data = ((char*)data) + 2;
 
     /* check strlen */
     for (n = 0; ((char*)data) [n] != 0; n ++)
